@@ -3,28 +3,41 @@ package Lingy::Eval;
 
 use Lingy::Common;
 
+use Exporter 'import';
+
+our @EXPORT = qw(
+    evaluate
+);
+
 # Lingy Special Forms:
-our %special_dispatch = (
-    'def!'              => \&special_def,
+my %special_dispatch = (
+    'def'               => \&special_def,
     'defmacro!'         => \&special_defmacro,
     'do'                => \&special_do,
     '.'                 => \&special_dot,
     'fn*'               => \&special_fn,
     'if'                => \&special_if,
+    'import*'           => \&special_import,
     'let*'              => \&special_let,
     'loop'              => \&special_loop,
+    'new'               => \&special_new,
     'recur'             => \&special_recur,
     'quasiquote'        => \&special_quasiquote,
     'quasiquoteexpand'  => \&special_quasiquoteexpand,
     'quote'             => \&special_quote,
     'try*'              => \&special_try,
+    'throw'             => \&special_throw,
     'var'               => \&special_var,
 );
 
+sub special_symbols {
+    keys %special_dispatch;
+}
+
 
 # Main eval functions:
 our $ENV;
-sub eval {
+sub evaluate {
     my ($ast, $env) = @_;
     $ENV = $env;
 
@@ -53,6 +66,9 @@ sub eval {
                 } elsif ($ref eq VECTOR) {
                     return special_vector($env, $fn, @args);
 
+                } elsif ($ref eq NIL) {
+                    return $args[0];
+
                 } else {
                     err "Can't use '$ref' object as function";
                 }
@@ -65,9 +81,9 @@ sub eval {
 sub eval_ast {
     my ($ast, $env) = @_;
     $ast->isa(LISTTYPE)
-        ? ref($ast)->new([ map Lingy::Eval::eval($_, $env), @$ast ]) :
+        ? ref($ast)->new([ map evaluate($_, $env), @$ast ]) :
     $ast->isa(HASHMAP)
-        ? ref($ast)->new([map Lingy::Eval::eval($_, $env), %$ast]) :
+        ? ref($ast)->new([map evaluate($_, $env), %$ast]) :
     $ast->isa(SYMBOL)
         ? $env->get($$ast) :
     $ast;
@@ -78,15 +94,17 @@ sub eval_ast {
 sub special_def {
     my ($ast, $env) = @_;
     my (undef, $a1, $a2) = @$ast;
+    $a1 // err "Too few arguments to def";
+    $a2 //= nil;
     err "Can't def a qualified symbol: '$a1'"
         if $a1 =~ m{./.};
-    return $env->ns_set($$a1, Lingy::Eval::eval($a2, $env));
+    return $env->ns_set($$a1, evaluate($a2, $env));
 }
 
 sub special_defmacro {
     my ($ast, $env) = @_;
     my (undef, $a1, $a2) = @$ast;
-    return $env->ns_set($$a1, macro(Lingy::Eval::eval($a2, $env)));
+    return $env->ns_set($$a1, MACRO->new(evaluate($a2, $env)));
 }
 
 sub special_do {
@@ -97,6 +115,7 @@ sub special_do {
     return ($ast, $env);
 }
 
+# TODO This function needs more attention and testing.
 sub special_dot {
     my ($ast, $env) = @_;
 
@@ -107,17 +126,30 @@ sub special_dot {
 
     ($target, @args) = map {
         $_->isa(LIST)
-            ? Lingy::Eval::eval($_, $env) : $_;
+            ? evaluate($_, $env) : $_;
     } ($target, @args);
 
     @args > 0 or err "Not enough args for . form";
 
     if (my $class = is_class_symbol($target)) {
-        $target = "${class}::${shift @args}";
-        @args = map { $_->isa(SYMBOL)
-            ? $env->get($_) : $_; } @args;
-        no strict 'refs';
-        return &{$target}(@args);
+        if (RT->is_lingy_class($class)) {
+            $target = "${class}::${shift @args}";
+            @args = map {
+                $_->isa(SYMBOL) ? $env->get($_) : $_;
+            } @args;
+            no strict 'refs';
+            return &{$target}(@args);
+        }
+
+        else {
+            my $method = shift(@args) or die;
+            $method->isa(SYMBOL) or die;
+            $method = $$method;
+            @args = map {
+                unbox_val( $_->isa(SYMBOL) ? $env->get($_) : $_ );
+            } @args;
+            return box_val $class->$method(@args);
+        }
     }
 
     $target = $env->get($target)
@@ -125,30 +157,26 @@ sub special_dot {
 
     if (@args) {
         my $member = shift(@args);
-        if ($target->isa(CLASS) or
-            $target->isa('Lingy::Namespace')
-        ) {
+        if ($target->isa(CLASS)) {
             @args = map { $_->isa(SYMBOL)
                 ? $env->get($_) : $_; } @args;
 
             if (not $target->can($member)) {
-                my $class = $target->isa('Lingy::Namespace')
-                    ? 'lingy.Namespace' : $target->NAME;
+                my $class = $target->NAME;
                 err "No matching field found: '$member' " .
                     "for class '$class'";
             }
             my $method = $$member;
             return $target->$method(@args);
         }
+
         # Test the unboxing of native call args:
         if ($target->can($member)) {
             my $method = $$member;
             @args = map {
-                (
-                    $_->isa(SYMBOL) ? $env->get($_) : $_
-                )->unbox;
+                unbox_val( $_->isa(SYMBOL) ? $env->get($_) : $_ );
             } @args;
-            return $target->$method(@args);
+            return box_val $target->$method(@args);
         }
     }
 
@@ -157,23 +185,56 @@ sub special_dot {
 
 sub special_fn {
     my ($ast, $env) = @_;
-    return function($ast, $env);
+    return FUNCTION->new($ast, $env);
 }
 
 sub special_if {
     my ($ast, $env) = @_;
     my (undef, $a1, $a2, $a3) = @$ast;
-    $ast = ${boolean(Lingy::Eval::eval($a1, $env))} ? $a2 :
+    $ast = ${BOOLEAN->new(evaluate($a1, $env))} ? $a2 :
         defined $a3 ? $a3 : nil;
     return ($ast, $env);
 }
 
+sub special_import {
+    my ($ast, $env) = @_;
+
+    my ($fn, @specs) = @$ast;
+
+    my $return = nil;
+
+    for my $spec (@specs) {
+        if (ref($spec) eq SYMBOL) {
+            $spec = list([$spec]);
+        }
+
+        err "Invalid import spec" unless
+            $spec->isa(LIST) and
+            @$spec > 0 and
+            not grep { ref($_) ne SYMBOL } @$spec;
+
+        my ($module_name, $imports) = @$spec;
+        my $name = $$module_name;
+        (my $module = $name) =~ s/\./::/g;
+        $module =~ s/lingy::lang::/Lingy::Lang::/;
+        eval "require $module; 1" or die $@;
+        my $class = RT->current_ns->{$name} =
+            CLASS->_new($name);
+        if ($module->can('new')) {
+            $return = $class;
+        }
+        # TODO - imports
+    }
+
+    return $return;
+}
+
 sub special_keyword {
     my ($env, $keyword, @args) = @_;
     err "Wrong number of args (${\ scalar @args}) passed to: '$keyword'"
         unless @args == 1 or @args == 2;
     my $map = shift @args;
-    Lingy::Eval::eval(
+    evaluate(
         list([
             symbol('get'),
             $map,
@@ -193,7 +254,7 @@ sub special_let {
     for (my $i = 0; $i < @$a1; $i += 2) {
         $env->set(
             ${$a1->[$i]},
-            Lingy::Eval::eval($a1->[$i+1], $env),
+            evaluate($a1->[$i+1], $env),
         );
     }
     $a2 = list([symbol('do'), @{$ast}[2..(@$ast-1)] ])
@@ -213,7 +274,7 @@ sub special_loop {
             push @$binds, ${$a1->[$i]};
             $env->set(
                 ${$a1->[$i]},
-                Lingy::Eval::eval($a1->[$i+1], $env),
+                evaluate($a1->[$i+1], $env),
             );
         }
         $a2 //= nil;
@@ -238,6 +299,11 @@ sub special_loop {
     return ($a2, $env);
 }
 
+sub special_new {
+    my ($ast, $env) = @_;
+    XXX @_, 'special_new not yet implemented';
+}
+
 sub special_quasiquote {
     my ($ast, $env) = @_;
     return (quasiquote($ast->[1]), $env);
@@ -257,8 +323,8 @@ sub special_recur {
     my ($ast, $env) = @_;
     $env->{RECUR} = 1;
     return (
-        list([symbol('loop'), vector([
-            map Lingy::Eval::eval($_, $env),
+        list([symbol('loop'), VECTOR->new([
+            map evaluate($_, $env),
                 @{$ast}[1..(@$ast-1)]])
         ]),
         $env,
@@ -269,10 +335,10 @@ sub special_try {
     my ($ast, $env) = @_;
     my (undef, $a1, $a2) = @$ast;
     local $@;
-    my $val = eval { Lingy::Eval::eval($a1, $env) };
+    my $val = eval { evaluate($a1, $env) };
     return $val unless $@;
     my $err = $@;
-    die ref($err) ? Lingy::Printer::pr_str($err) : $err
+    die ref($err) ? RT->printer->pr_str($err) : $err
         unless defined $a2;
     err "Invalid 'catch' clause" unless
         $a2 and $a2->isa(LISTTYPE) and
@@ -294,12 +360,20 @@ sub special_try {
     );
 }
 
+sub special_throw {
+    my ($ast, $env) = @_;
+    require Carp;
+    Carp::confess(
+        evaluate($ast->[1], $env),
+    );
+}
+
 sub special_var {
     my ($ast, $env) = @_;
     my (undef, $a1) = @$ast;
     $env->get($a1, 1)
         or err "Unable to resolve var: '$a1' in this context";
-    return var($a1);
+    return VAR->new($a1);
 }
 
 sub special_vector {
@@ -307,7 +381,7 @@ sub special_vector {
     err "Wrong number of args (${\ scalar @args}) passed to: " .
         "'lingy.lang.Vector'"
         unless @args == 1;
-    Lingy::Eval::eval(
+    evaluate(
         list([
             symbol('nth'),
             $vector,
@@ -349,14 +423,35 @@ sub macroexpand {
             $member = symbol(substr($$member, 1));
             return list([symbol('.'), $instance, $member, @rest]);
         }
-#         if ($sym =~ /^(.*)\.$/) {
-#             XXX my $class = $Lingy::RT::class->{String};
-#         }
+        if ($sym =~ /^($namespace_re)\.$/) {
+            my (undef, @args) = @$ast;
+            # XXX Should expand to (new Foo.Bar 1 2 3)
+            return list([
+                symbol('.'),
+                symbol($1),
+                symbol('new'),
+                @args,
+            ]);
+        }
+        if ($sym =~ m{^($namespace_re)/($symbol_re)$}) {
+            my $namespace = $1;
+            my $sym_name = $2;
+            (my $class = $namespace) =~ s/\./::/g;
+            if ($class->can('new')) {
+                my (undef, @args) = @$ast;
+                return list([
+                    symbol('.'),
+                    symbol($namespace),
+                    symbol($sym_name),
+                    @args,
+                ]);
+            }
+        }
         if (($call = $env->get($sym, 1)) and
             ref($call) eq MACRO
         ) {
             # expand macro call form
-            $ast = Lingy::Eval::eval($call->(@{$ast}[1..(@$ast-1)]));
+            $ast = evaluate($call->(@{$ast}[1..(@$ast-1)]));
         } else {
             last;
         }
