@@ -1,11 +1,12 @@
 use strict; use warnings;
 package Lingy::Lang::Macro;
 
-use base 'Lingy::Lang::Class';
+use Lingy::Common;
+use base CLASS;
 
 sub new {
     my ($class, $function) = @_;
-    XXX $function unless ref($function) eq 'Lingy::Lang::Function';
+    XXX $function unless ref($function) eq FUNCTION;
     bless sub { goto &$function }, $class;
 }
 
